package com.qgs.dc.s7.my.s7connector.api.utils;

/**
 * @Desc: ""
 * @Author: caixiang
 * @DATE: 2021/12/16 9:08
 */

import cn.hutool.core.util.ByteUtil;

import java.io.UnsupportedEncodingException;
import java.nio.ByteOrder;
import java.nio.charset.StandardCharsets;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Stack;

public class ByteUtils {


    public static double bytes2Double(byte[] arr) {
        long value = 0;
        for (int i = 0; i < 8; i++) {
            value |= ((long) (arr[i] & 0xff)) << (8 * i);
        }

        return Double.longBitsToDouble(value);
    }

    public static Boolean toBoolean(byte[] bytes){
        if(bytes.length ==0){
            return null;
        }
        if(bytes[0] == 1){
            return true;
        }else {
            return false;
        }
    }
    public static Boolean toBoolean(byte bytes){
        if(bytes == 1){
            return true;
        }else {
            return false;
        }
    }

    public static String addDate(String timeParam, Long day) throws ParseException {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd"); // 日期格式
        Date date = dateFormat.parse(timeParam); // 指定日期

        long time = date.getTime(); // 得到指定日期的毫秒数
        day = day * 24 * 60 * 60 * 1000; // 要加上的天数转换成毫秒数
        time += day; // 相加得到新的毫秒数
        Date newDate = new Date(time);
        return dateFormat.format(newDate); // 将毫秒数转换成日期
    }


    public static byte[] invert(byte[] a){
        byte[] b = new byte[a.length];
        Stack<Byte> st = new Stack<Byte>();
        for(int i=0;i<a.length;i++){
            st.push(a[i]);
        }
        for(int i=0;i<a.length;i++){
            b[i] = st.pop();
        }
        return b;
    }


    /**
     * 1个字节byte[] 转成有符号的Integer
     * */
    public static Integer toInt(byte bytes) {
        return Integer.valueOf(Byte.toString(bytes));
    }
    /**
     * 1个字节byte[] 转成无符号的Integer
     * */
    public static Integer toUInt(byte bytes) {
        int i = Byte.toUnsignedInt(bytes);
        return Integer.valueOf(i);
    }


    /**
     * 2个字节byte[] 转成有符号的Integer
     * 默认大端
     * */
    public static Integer toInt(byte byte1,byte byte2) {
        byte[] bytes = new byte[2];
        bytes[0] = byte1;
        bytes[1] = byte2;

        return Integer.valueOf(Short.toString(ByteUtil.bytesToShort(bytes, ByteOrder.BIG_ENDIAN)));
    }
    /**
     * 2个字节byte[] 转成无符号的Integer
     * 默认大端
     * */
    public static Integer toUInt(byte byte1,byte byte2) {
        byte[] bytes = new byte[2];
        bytes[0] = byte1;
        bytes[1] = byte2;
        short i = ByteUtil.bytesToShort(bytes, ByteOrder.BIG_ENDIAN);
        return Short.toUnsignedInt(i);
    }


    /**
     * 4个字节byte[] 转成有符号的Integer
     * 默认大端
     * */
    public static Integer toInt(byte byte1,byte byte2,byte byte3,byte byte4) {
        byte[] bytes = new byte[4];
        bytes[0] = byte1;
        bytes[1] = byte2;
        bytes[2] = byte3;
        bytes[3] = byte4;
        return ByteUtil.bytesToInt(bytes, ByteOrder.BIG_ENDIAN);
    }
    /**
     * 4个字节byte[] 转成无符号的Long（因为如果首位为1 Integer就不满足长度了）
     * 默认大端
     * */
    public static Long toUInt(byte byte1,byte byte2,byte byte3,byte byte4) {
        byte[] bytes = new byte[8];
        bytes[0] = 0;
        bytes[1] = 0;
        bytes[2] = 0;
        bytes[3] = 0;
        bytes[4] = byte1;
        bytes[5] = byte2;
        bytes[6] = byte3;
        bytes[7] = byte4;
        long l = ByteUtil.bytesToLong(bytes, ByteOrder.BIG_ENDIAN);
        return l;
    }

    /**
     *
     * 4个字节byte[] 转成有符号的Double
     * 默认大端
     * */
    public static Float forReal(byte[] bytes) {
        return Float.intBitsToFloat(toInt(bytes[0],bytes[1],bytes[2],bytes[3]));
    }
    /**
     *
     * 8个字节byte[] 转成有符号的Double
     * 默认大端
     * */
    public static Double forLReal(byte[] bytes) {
        return ByteUtil.bytesToDouble(bytes, ByteOrder.BIG_ENDIAN);
    }

    /**
     *
     * 8个字节byte[] 转成有符号的Double
     * 默认大端
     * */
    public static Double lrealbytesToDouble(byte[] bytes) {
        return ByteUtil.bytesToDouble(bytes, ByteOrder.BIG_ENDIAN);
    }
    public static byte[] lrealToBytes(double doubleValue) {
        return ByteUtil.doubleToBytes(doubleValue, ByteOrder.BIG_ENDIAN);
        //return double2Bytes(doubleValue);
    }

//    public static String toChar(byte[] b) throws UnsupportedEncodingException {
//        String ascii = new String(b, Charset.forName("UTF-8"));
//        return ascii;
//    }
    public static Character toChar(byte[] b) throws UnsupportedEncodingException {
        if(b.length==1){
            return toChar(b[0]);
        }
        return byteToChar(b);
    }
    public static Character toChar(byte b) throws UnsupportedEncodingException {
        return byteToChar(b);
    }
//    public static String toChar(byte b) throws UnsupportedEncodingException {
//        byte[] bs = new byte[1];
//        bs[0] = b;
//        String ascii = new String(bs, "ascii");
//        return ascii;
//    }

    /**
     * return null 代表返回传入参数不正确str 取的length太小
     * */
    public static String toStr(byte[] b) throws UnsupportedEncodingException {
        Integer length = Byte.toUnsignedInt(b[1]);
        if(length>(b.length-2)){
            return null;
        }
        byte[] content = new byte[b.length-2];
        for(int i=0;i<length;i++){
            content[i] = b[i+2];
        }
        String ascii = new String(content, StandardCharsets.UTF_8);
       // String s = new String(content);
        return ascii;
    }

    public static List<Boolean> toBoolArray(byte[] b) throws UnsupportedEncodingException {
        List<Boolean> res = new ArrayList<>();
        for(int i=0;i<b.length;i++){
            res.add(toBoolean(b[i]));
        }
        return res;
    }

    public static List<Byte> toByteArray(byte[] b) throws UnsupportedEncodingException {
        List<Byte> res = new ArrayList<>();
        for(int i=0;i<b.length;i++){
            res.add((b[i]));
        }
        return res;
    }

    public static List<Character> toCharArray(byte[] b) throws UnsupportedEncodingException {
        List<Character> res = new ArrayList<>();
        for(int i=0;i<b.length;i++){
            res.add(toChar(b[i]));
        }
        return res;
    }

    //toWord
    /**
     * 默认：word => 有符号的整形
     * */
    public static List<Integer> toWordArray(byte[] b) throws UnsupportedEncodingException {
        List<Integer> res = new ArrayList<>();
        int i=0;
        while ((i+2)<=b.length){
            res.add(
                    toInt(b[i],b[i+1])
            );
            i+=2;
        }
        return res;
    }

    //toDWord
    /**
     * 默认：dword => 有符号的整形
     * */
    public static List<Integer> toDWordArray(byte[] b) throws UnsupportedEncodingException {
        List<Integer> res = new ArrayList<>();
        int i=0;
        while ((i+4)<=b.length){
            res.add(
                    toInt(b[i],b[i+1],b[i+2],b[i+3])
            );
            i+=4;
        }
        return res;
    }
    /**
     *
     * 4个字节byte[] 转成有符号的Double
     * 默认大端
     * */
    public static Float realbytesToFloat(byte[] bytes) {

        return Float.intBitsToFloat(toInt(bytes[0],bytes[1],bytes[2],bytes[3]));
    }
    public static byte[] realToBytes(Float f) {
        return invert(float2byte(f));
    }
    /**
     * 浮点转换为字节
     *
     * @param f
     * @return
     */
    private static byte[] float2byte(float f) {

        // 把float转换为byte[]
        int fbit = Float.floatToIntBits(f);

        byte[] b = new byte[4];
        for (int i = 0; i < 4; i++) {
            b[i] = (byte) (fbit >> (24 - i * 8));
        }

        // 翻转数组
        int len = b.length;
        // 建立一个与源数组元素类型相同的数组
        byte[] dest = new byte[len];
        // 为了防止修改源数组，将源数组拷贝一份副本
        System.arraycopy(b, 0, dest, 0, len);
        byte temp;
        // 将顺位第i个与倒数第i个交换
        for (int i = 0; i < len / 2; ++i) {
            temp = dest[i];
            dest[i] = dest[len - i - 1];
            dest[len - i - 1] = temp;
        }

        return dest;

    }

    //toUInt
    /**
     * USInt 无符号整形 1个字节  =》 Integer
     * */
    public static List<Integer> toUSIntArray(byte[] b) throws UnsupportedEncodingException {
        List<Integer> res = new ArrayList<>();
        for(int i=0;i<b.length;i++){
            res.add(toUInt(b[i]));
        }
        return res;
    }
    /**
     * UInt 无符号整形 2个字节  =》 Integer
     * */
    public static List<Integer> toUIntArray(byte[] b) throws UnsupportedEncodingException {
        List<Integer> res = new ArrayList<>();
        int i=0;
        while ((i+2)<=b.length){
            res.add(
                    toUInt(b[i],b[i+1])
            );
            i+=2;
        }
        return res;
    }
    /**
     * UDInt 无符号整形 4个字节  =》 Long
     * */
    public static List<Long> toUDIntArray(byte[] b) throws UnsupportedEncodingException {
        List<Long> res = new ArrayList<>();
        int i=0;
        while ((i+4)<=b.length){
            res.add(
                    toUInt(b[i],b[i+1],b[i+2],b[i+3])
            );
            i+=4;
        }
        return res;
    }

    /**
     * SInt 无符号整形 1个字节  =》 Integer
     * */
    public static List<Integer> toSIntArray(byte[] b) throws UnsupportedEncodingException {
        List<Integer> res = new ArrayList<>();
        for(int i=0;i<b.length;i++){
            res.add(toInt(b[i]));
        }
        return res;
    }
    /**
     * Int 无符号整形 2个字节  =》 Integer
     * */
    public static List<Integer> toIntArray(byte[] b) throws UnsupportedEncodingException {
        List<Integer> res = new ArrayList<>();
        int i=0;
        while ((i+2)<=b.length){
            res.add(
                    toInt(b[i],b[i+1])
            );
            i+=2;
        }
        return res;
    }
    /**
     * DInt 无符号整形 4个字节  =》 Integer
     * */
    public static List<Integer> toDIntArray(byte[] b) throws UnsupportedEncodingException {
        List<Integer> res = new ArrayList<>();
        int i=0;
        while ((i+4)<=b.length){
            res.add(
                    toInt(b[i],b[i+1],b[i+2],b[i+3])
            );
            i+=4;
        }
        return res;
    }

    /**
     * sint(1个字节)  =》 byte[]
     * 默认大端模式
     * */
    public static byte[] sintToBytes(Integer i){
        byte[] res = new byte[1];
        res[0] = Byte.valueOf(i.toString());
        return res;
    }
    /**
     * sintArray(1个字节)  =》 byte[]
     *
     * */
    public static byte[] sintArrayToBytes(int[] sintArray) {
        byte[] res = new byte[sintArray.length];
        for(int i=0;i<sintArray.length;i++){
            int i1 = sintArray[i];
            res[i] = sintToBytes(i1)[0];
        }
        return res;
    }

    /**
     * int(2个字节)  =》 byte[]
     * 默认大端模式
     * */
    public static byte[] intToBytes(Integer i) {
        Number shortNumber = Short.valueOf(i.toString());
        return ByteUtil.numberToBytes(shortNumber,ByteOrder.BIG_ENDIAN);
    }
    /**
     * intArray(2个字节)  =》 byte[]
     *
     * */
    public static byte[] intArrayToBytes(int[] intArray) {
        byte[] res = new byte[intArray.length*2];
        for(int i=0 ; i< intArray.length ; i++){
            byte[] bytes = intToBytes(intArray[i]);
            res[i*2] = bytes[0];
            res[i*2+1] = bytes[1];
        }
        return res;
    }

    /**
     * int(2个字节)  =》 byte[]
     * 默认大端模式
     * */
    public static byte[] dintToBytes(Integer i) {
        Number intNumber = i;
        return ByteUtil.numberToBytes(intNumber,ByteOrder.BIG_ENDIAN);
    }
    /**
     * intArray(2个字节)  =》 byte[]
     *
     * */
    public static byte[] dintArrayToBytes(int[] dintArray) {
        byte[] res = new byte[dintArray.length*4];
        for(int i=0 ; i< dintArray.length ; i++){
            byte[] bytes = dintToBytes(dintArray[i]);
            res[i*4] = bytes[0];
            res[i*4+1] = bytes[1];
            res[i*4+2] = bytes[2];
            res[i*4+3] = bytes[3];

        }
        return res;
    }

    /**
     * sint(1个字节)  =》 byte[]
     * 默认大端模式
     * */
    public static byte[] usintToBytes(Integer i){
        if(i<0){
            return null;
        }
        byte[] res = new byte[1];
        res[0] = Byte.valueOf(i.toString());
        return res;
    }
    /**
     * usintArrayToBytes  =》 byte[]
     * 默认大端模式
     * */
    public static byte[] usintArrayToBytes(int[] usintArray) {
        byte[] res = new byte[usintArray.length];
        for(int i=0 ; i< usintArray.length ; i++){
            byte[] bytes = usintToBytes(usintArray[i]);
            if(bytes == null){
                return null;
            }
            res[i] = bytes[0];
        }
        return res;
    }


    /**
     * int(2个字节)  =》 byte[]
     * 默认大端模式
     * */
    public static byte[] uintToBytes(Integer i) {
        if(i<0){
            return null;
        }
        Number shortNumber = Short.valueOf(i.toString());
        return ByteUtil.numberToBytes(shortNumber,ByteOrder.BIG_ENDIAN);
    }
    public static byte[] uintArrayToBytes(int[] uintArray) {
        byte[] res = new byte[uintArray.length*2];
        for(int i=0 ; i< uintArray.length ; i++){
            byte[] bytes = uintToBytes(uintArray[i]);
            if(bytes == null){
                return null;
            }
            res[i*2] = bytes[0];
            res[i*2+1] = bytes[1];
        }
        return res;
    }

    /**
     * int(2个字节)  =》 byte[]
     * 默认大端模式
     * */
    public static byte[] udintToBytes(Integer i) {
        if(i<0){
            return null;
        }
        Number intNumber = i;
        return ByteUtil.numberToBytes(intNumber,ByteOrder.BIG_ENDIAN);
    }
    public static byte[] udintArrayToBytes(int[] udintArray) {
        byte[] res = new byte[udintArray.length*4];
        for(int i=0 ; i< udintArray.length ; i++){
            byte[] bytes = udintToBytes(udintArray[i]);
            if(bytes == null){
                return null;
            }
            res[i*4] = bytes[0];
            res[i*4+1] = bytes[1];
            res[i*4+2] = bytes[2];
            res[i*4+3] = bytes[3];
        }
        return res;
    }

    /**
     * boolean  =》 byte[]
     * 默认大端模式
     * */
    public static byte[] boolToBytes(Boolean bool) {
        byte[] res = new byte[1];
        if(bool){
            res[0] = 1;
            return res;
        }else {
            res[0] = 0;
            return res;
        }
    }
    /**
     * booleanArray  =》 byte[]
     * */
    public static byte[] booleanArrayToBytes(boolean[] boolArray) {
        byte[] res = new byte[boolArray.length];
        for(int i=0 ; i<boolArray.length ; i++){
            boolean b = boolArray[i];
            if(b){
                res[i] = 1;
            }else {
                res[i] = 0;
            }
        }
        return res;
    }
    /**
     * charArray  =》 byte[]
     * */
    public static byte[] charArrayToBytes(char[] charArray) {
        byte[] res = new byte[charArray.length];
        for(int i=0 ; i< charArray.length ; i++){
            byte[] bytes = charToByte(charArray[i]);
            if(bytes[0] != 0){
                return null;
            }else {
                res[i] = bytes[1];
            }
        }
        return res;
    }





    /**
     * byte  =》 byte[]
     * 默认大端模式
     * */
    public static byte[] setbytes(Byte bytes) {
        byte[] res = new byte[1];
        res[0] = bytes;
        return res;
    }

    /**
     * word  =》 byte[]
     * 默认大端模式
     * tip: 目前只支持 带符号的十进制 ==》 word(2个字节)
     * */
    public static byte[] wordToBytes(Short word) {
        return ByteUtil.shortToBytes(word,ByteOrder.BIG_ENDIAN);
    }

    /**
     * wordArray  =》 byte[]
     * */
    public static byte[] wordArrayToBytes(short[] shortArray) {
        byte[] res = new byte[shortArray.length*2];
        for(int i=0 ; i< shortArray.length ; i++){
            byte[] bytes = wordToBytes(shortArray[i]);
            res[i*2] = bytes[0];
            res[i*2+1] = bytes[1];
        }
        return res;
    }

    /**
     * dword  =》 byte[]
     * 默认大端模式
     * tip: 目前只支持 带符号的十进制 ==》 word(4个字节)
     * */
    public static byte[] dwordToBytes(Integer dword) {
        return ByteUtil.intToBytes(dword,ByteOrder.BIG_ENDIAN);
    }

    /**
     * dwordArray  =》 byte[]
     * */
    public static byte[] dwordArrayToBytes(int[] intArray) {
        byte[] res = new byte[intArray.length*4];
        for(int i=0 ; i< intArray.length ; i++){
            byte[] bytes = dwordToBytes(intArray[i]);
            res[i*4] = bytes[0];
            res[i*4+1] = bytes[1];
            res[i*4+2] = bytes[2];
            res[i*4+3] = bytes[3];
        }
        return res;
    }

    /**
     * char(1字节)  =》 byte[]
     *
     * */
    public static byte[] charToBytes(Character c) {
        return String.valueOf(c).getBytes();
    }
    public static byte[] wcharToBytes(Character c) {
        return charToByte(c);
    }

    public static byte[] strToBytes(String s) {
        byte[] bytes = s.getBytes(StandardCharsets.UTF_8);
        byte[] res = new byte[bytes.length+2];
        res[0] = -2;
        res[1] = Integer.valueOf(bytes.length).byteValue();
        for(int i=0;i<bytes.length;i++){
           res[i+2] = bytes[i];
        }


        return res;
    }



    //for private

    private static byte[] charToByte(char c) {
        byte[] b = new byte[2];
        b[0] = (byte) ((c & 0xFF00) >> 8);
        b[1] = (byte) (c & 0xFF);
        return b;
    }
    private static char byteToChar(byte[] b) {
        char c = (char) (((b[0] & 0xFF) << 8) | (b[1] & 0xFF));
        return c;
    }
    private static char byteToChar(byte b) {
        char c = (char) (b & 0xFF);
        return c;
    }


}
