package com.qgs.dc.s7.my.s7connector.enmuc;

import com.qgs.dc.s7.my.s7connector.api.S7Connector;
import com.qgs.dc.s7.my.s7connector.api.factory.S7ConnectorFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

/**
 * @Desc: ""
 * @Author: caixiang
 * @DATE: 2022/1/15 13:01
 */
public enum S7Client {
    //TODO 步骤1  这里是配置多PLC 的，，，有多个plc 就在这里配置一个枚举类
    S7_1200("192.168.0.51",0,0,3,PlcVarActual.HeartBeat)
    //后续 在这里扩展 多PLC应用。




    ;
    private String host;
    //默认 0   机架号
    private Integer rack;
    //默认 0
    private Integer slot;

    //心跳变量，如果plc没有让电控的人加一个，这个是必填的
    private PlcVarActual heartBeat;

    private List<S7Connector> connections;
    //coreSize 是线程池的大小
    private Integer coreSize;
    //pickOne 就是一个初始化 的轮询取余值
    private int pickOne;
    private static final Logger logger = LoggerFactory.getLogger(S7Client.class);
    //coreSize 是线程池的数量
    S7Client(String host, Integer rack, Integer slot,Integer coreSize,PlcVarActual heartBeat){
        this.host = host;
        this.rack = rack;
        this.slot = slot;
        this.pickOne = 0;

        this.coreSize = coreSize;
        this.heartBeat = heartBeat;
        connections = new ArrayList<>();
        connectionPool();

        ping();

        check_ping();
    }



    private ScheduledExecutorService executor = Executors.newScheduledThreadPool(1);
    private ScheduledExecutorService ping_fail_check = Executors.newScheduledThreadPool(1);


    public S7Connector getConnector() {
        int size = connections.size();
        S7Connector s7Connector = connections.get((pickOne + size) % size);
        pickOne+=1;
        pickOne = (pickOne)%size;
        return s7Connector;
    }
    public S7Connector connect(String host,Integer rack,Integer slot ){
        try {
            S7Connector connector = S7ConnectorFactory
                    .buildTCPConnector()
                    .withHost(host)
                    .withRack(rack) //optional   rack 是机架号
                    .withSlot(slot) //optional   slot 是插槽号
                    .build();
            return connector;
        }catch (Exception e){
            logger.info("创建S7Connector  连接失败");
            return null;
        }

    }

    private void connectionPool(){
        for(int i=0;i<coreSize;i++){
            connections.add(connect(host,rack,slot));
        }
        //todo 在plc上新增一个 变量来解决 心跳问题 okok
    }

    private void check_ping(){
        ping_fail_check.execute(new Runnable() {
            @Override
            public void run() {
                while (true){
                    if(connections.size()!=coreSize){
                        int c = coreSize-connections.size();
                        for(int z=0;z<c;z++){
                            S7Connector connect = connect(host, rack, slot);
                            if(connect!=null){
                                connections.add(connect);
                            }else {
                                logger.info("check_ping   断线重连出现异常。。");
                            }

                        }
                    }
                    try {
                        Thread.sleep(10000);
                    }catch (Exception e){
                        logger.info(e.getMessage());
                    }
                }
            }
        });
    }

    private void ping(){
       executor.execute(new Runnable() {
            @Override
            public void run() {
                while (true){
                    for(int i=0;i<coreSize;i++){
                        S7Connector connector = getConnector();
                        try {
                            //只要没有报异常  都是通讯正常的。
                            byte[] read = connector.read(
                                    heartBeat.getArea(),
                                    heartBeat.getAreaNumber(),
                                    heartBeat.getType().getTransportSize().getSizeInBytes(),
                                    heartBeat.getByteOffset(),
                                    heartBeat.getBitOffset(),
                                    heartBeat.getType().getTransportSize());
                            System.out.println(connector.hashCode()+" :  ping");
                            Thread.sleep(100);
                        }catch (Exception e){
                            System.out.println(connector.hashCode()+" :  connection error");
                            logger.info(e.getMessage());
                            //先把 socket close掉
                            try {
                                connector.close();
                                connections.remove(connector);
                                //如果是网络波动照成的socket断开。 等个1S 再重连试试
                                Thread.sleep(100);
                            }catch (Exception ee){
                                logger.info(ee.getMessage());
                            }


                            //todo 把之前的连接close 掉，然后新增一个连接到connections

                            if(connections.size()!=coreSize){
                                int c = coreSize-connections.size();
                                for(int z=0;z<c;z++){
                                    S7Connector connect = connect(host, rack, slot);
                                    if(connect!=null){
                                        connections.add(connect);
                                    }else {
                                        logger.info("断线重连出现异常。。");
                                    }

                                }
                            }
                        }
                    }

                    try {
                        Thread.sleep(30000);
                    }catch (Exception e){
                        e.printStackTrace();
                        logger.info(e.getMessage());
                    }
                }
            }
        });
    }
}